@@ -5,7 +5,6 @@ namespace ledger {
 void format_emacs_transactions::write_entry(entry_t& entry)
 {
   int idx = entry.src_idx;
-
   for (strings_list::iterator i = entry.journal->sources.begin();
        i != entry.journal->sources.end();
        i++)
@@ -14,22 +13,10 @@ void format_emacs_transactions::write_entry(entry_t& entry)
       break;
     }
 
-  out << (((unsigned long)entry.beg_pos) + 1) << " ";
+  out << ((unsigned long)entry.beg_line) << " ";
 
-  switch (entry.state) {
-  case entry_t::CLEARED:
-    out << "t ";
-    break;
-  case entry_t::PENDING:
-    out << "pending ";
-    break;
-  case entry_t::UNCLEARED:
-    out << "nil ";
-    break;
-  }
-
-  out << "(" << (entry.date / 65536) << " "
-      << (entry.date % 65536) << " 0) ";
+  std::time_t date = entry.date().when;
+  out << "(" << (date / 65536) << " " << (date % 65536) << " 0) ";
 
   if (entry.code.empty())
     out << "nil ";
@@ -60,8 +47,22 @@ void format_emacs_transactions::operator()(transaction_t& xact)
       out << "\n";
     }
 
-    out << "  (\"" << xact.account->fullname() << "\" \""
+    out << "  (" << ((unsigned long)xact.beg_line) << " ";
+    out << "\"" << xact_account(xact)->fullname() << "\" \""
 	<< xact.amount << "\"";
+
+    switch (xact.state) {
+    case transaction_t::CLEARED:
+      out << " t";
+      break;
+    case transaction_t::PENDING:
+      out << " pending";
+      break;
+    default:
+      out << " nil";
+      break;
+    }
+
     if (xact.cost)
       out << " \"" << *xact.cost << "\"";
     else if (! xact.note.empty())
@@ -77,26 +78,3 @@ void format_emacs_transactions::operator()(transaction_t& xact)
 }
 
 } // namespace ledger
-
-#ifdef USE_BOOST_PYTHON
-
-#include <boost/python.hpp>
-
-using namespace boost::python;
-using namespace ledger;
-
-void export_emacs()
-{
-  typedef
-    pystream_handler_wrap<format_emacs_transactions, transaction_t>
-    format_emacs_transactions_wrap;
-
-  class_< format_emacs_transactions_wrap, bases<item_handler<transaction_t> > >
-    ("FormatEmacsTransactions",
-     init<PyObject *>()[with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_emacs_transactions_wrap::flush)
-    .def("__call__", &format_emacs_transactions_wrap::operator())
-    ;
-}
-
-#endif // USE_BOOST_PYTHON
