@@ -615,6 +615,9 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 	case transaction_t::PENDING:
 	  out << "! ";
 	  break;
+	default:
+	  out << "";
+	  break;
 	}
       }
       break;
@@ -630,6 +633,9 @@ void format_t::format(std::ostream& out_str, const details_t& details) const
 	  case transaction_t::PENDING:
 	    out << "! ";
 	    break;
+	  default:
+	    out << "";
+	    break;
 	  }
       }
       break;
