@@ -2,13 +2,20 @@
 #include "journal.h"
 #include "format.h"
 #include "error.h"
+#include "acconf.h"
 
 #include <iostream>
 #include <sstream>
 #include <cstring>
 
 extern "C" {
-#include <xmlparse.h>           // expat XML parser
+#if defined(HAVE_EXPAT)
+#include <expat.h>           // expat XML parser
+#elif defined(HAVE_XMLPARSE)
+#include <xmlparse.h>        // expat XML parser
+#else
+#error "No XML parser library defined."
+#endif
 }
 
 namespace ledger {
@@ -34,7 +41,17 @@ static account_comm_map	account_comms;
 static unsigned int	count;
 static std::string	have_error;
 
-static enum {
+static std::istream *   instreamp;
+static unsigned int     offset;
+static XML_Parser       parser;
+static std::string      path;
+static unsigned int     src_idx;
+static istream_pos_type beg_pos;
+static unsigned long    beg_line;
+
+static transaction_t::state_t curr_state;
+
+static enum action_t {
   NO_ACTION,
   ACCOUNT_NAME,
   ACCOUNT_ID,
@@ -65,10 +82,8 @@ static void startElement(void *userData, const char *name, const char **atts)
     action = ACCOUNT_ID;
   else if (std::strcmp(name, "act:parent") == 0)
     action = ACCOUNT_PARENT;
-  else if (std::strcmp(name, "gnc:commodity") == 0) {
-    assert(! curr_comm);
-    curr_comm = new commodity_t("");
-  }
+  else if (std::strcmp(name, "gnc:commodity") == 0)
+    curr_comm = NULL;
   else if (std::strcmp(name, "cmdty:id") == 0)
     action = COMM_SYM;
   else if (std::strcmp(name, "cmdty:name") == 0)
@@ -115,26 +130,74 @@ static void endElement(void *userData, const char *name)
     curr_account = NULL;
   }
   else if (std::strcmp(name, "gnc:commodity") == 0) {
-    assert(curr_comm);
-    commodity_t::add_commodity(curr_comm);
     curr_comm = NULL;
   }
   else if (std::strcmp(name, "gnc:transaction") == 0) {
     assert(curr_entry);
+
+    // Add the new entry (what gnucash calls a 'transaction') to the
+    // journal
     if (! curr_journal->add_entry(curr_entry)) {
       print_entry(std::cerr, *curr_entry);
       have_error = "The above entry does not balance";
       delete curr_entry;
     } else {
+      curr_entry->src_idx  = src_idx;
+      curr_entry->beg_pos  = beg_pos;
+      curr_entry->beg_line = beg_line;
+      curr_entry->end_pos  = instreamp->tellg();
+      curr_entry->end_line = XML_GetCurrentLineNumber(parser) - offset;
       count++;
     }
+
+    // Clear the relevant variables for the next run
     curr_entry = NULL;
+    entry_comm = NULL;
   }
+  else if (std::strcmp(name, "trn:split") == 0) {
+    transaction_t * xact = curr_entry->transactions.back();
+
+    // Identify the commodity to use for the value of this
+    // transaction.  The quantity indicates how many times that value
+    // the transaction is worth.
+    amount_t value;
+    commodity_t * default_commodity = NULL;
+    account_comm_map::iterator ac = account_comms.find(xact->account);
+    if (ac != account_comms.end())
+      default_commodity = (*ac).second;
+
+    if (default_commodity) {
+      curr_quant.set_commodity(*default_commodity);
+      value = curr_quant.round();
+
+      if (curr_value.commodity() == *default_commodity)
+	curr_value = value;
+    } else {
+      value = curr_quant;
+    }
+
+    xact->state  = curr_state;
+    xact->amount = value;
+    if (value != curr_value)
+      xact->cost = new amount_t(curr_value);
+
+    xact->beg_pos  = beg_pos;
+    xact->beg_line = beg_line;
+    xact->end_pos  = instreamp->tellg();
+    xact->end_line = XML_GetCurrentLineNumber(parser) - offset;
+
+    // Clear the relevant variables for the next run
+    curr_state = transaction_t::UNCLEARED;
+    curr_value = amount_t();
+    curr_quant = amount_t();
+  }
+
   action = NO_ACTION;
 }
 
 
-static amount_t convert_number(const std::string& number)
+static amount_t convert_number(const std::string& number,
+			       int * precision = NULL)
 {
   const char * num = number.c_str();
 
@@ -145,6 +208,9 @@ static amount_t convert_number(const std::string& number)
     amount_t amt(numer_str);
     amount_t den(denom_str);
 
+    if (precision)
+      *precision = denom_str.length() - 1;
+
     if (! den) {
       have_error = "Denominator in entry is zero!";
       return amt;
@@ -176,43 +242,38 @@ static void dataHandler(void *userData, const char *s, int len)
     break;
   }
 
-  case COMM_SYM:
-    if (curr_comm) {
-      curr_comm->set_symbol(std::string(s, len));
-    }
-    else if (curr_account) {
+  case COMM_SYM: {
       std::string symbol(s, len);
-      commodity_t * comm = commodity_t::find_commodity(symbol, true);
-      if (symbol != "$" && symbol != "USD")
-	comm->flags |= COMMODITY_STYLE_SEPARATED;
-      account_comms.insert(account_comm_pair(curr_account, comm));
-    }
-    else if (curr_entry) {
-      std::string symbol(s, len);
-      entry_comm = commodity_t::find_commodity(symbol, true);
-      if (symbol != "$" && symbol != "USD")
-	entry_comm->flags |= COMMODITY_STYLE_SEPARATED;
-    }
+    if (symbol == "USD") symbol = "$";
+
+    curr_comm = commodity_t::find_or_create(symbol);
+    assert(curr_comm);
+
+    if (symbol != "$")
+      curr_comm->add_flags(COMMODITY_STYLE_SEPARATED);
+
+    if (curr_account)
+      account_comms.insert(account_comm_pair(curr_account, curr_comm));
+    else if (curr_entry)
+      entry_comm = curr_comm;
     break;
+  }
 
   case COMM_NAME:
-    curr_comm->name = std::string(s, len);
+    curr_comm->set_name(std::string(s, len));
     break;
 
   case COMM_PREC:
-    curr_comm->precision = len - 1;
+    curr_comm->set_precision(len - 1);
     break;
 
   case ENTRY_NUM:
     curr_entry->code = std::string(s, len);
     break;
 
-  case ENTRY_DATE: {
-    struct tm when;
-    strptime(std::string(s, len).c_str(), "%Y-%m-%d %H:%M:%S %z", &when);
-    curr_entry->date = std::mktime(&when);
+  case ENTRY_DATE:
+    curr_entry->_date = std::string(s, len);
     break;
-  }
 
   case ENTRY_DESC:
     curr_entry->payee = std::string(s, len);
@@ -220,16 +281,23 @@ static void dataHandler(void *userData, const char *s, int len)
 
   case XACT_STATE:
     if (*s == 'y')
-      curr_entry->state = entry_t::PENDING;
+      curr_state = transaction_t::CLEARED;
+    else if (*s == 'n')
+      curr_state = transaction_t::UNCLEARED;
     else
-      curr_entry->state = entry_t::CLEARED;
+      curr_state = transaction_t::PENDING;
     break;
 
-  case XACT_VALUE:
+  case XACT_VALUE: {
+    int precision;
     assert(entry_comm);
-    curr_value = convert_number(std::string(s, len));
+    curr_value = convert_number(std::string(s, len), &precision);
     curr_value.set_commodity(*entry_comm);
+
+    if (precision > entry_comm->precision())
+      entry_comm->set_precision(precision);
     break;
+  }
 
   case XACT_QUANTITY:
     curr_quant = convert_number(std::string(s, len));
@@ -247,25 +315,6 @@ static void dataHandler(void *userData, const char *s, int len)
       have_error = (std::string("Could not find account ") +
 		    std::string(s, len));
     }
-
-    amount_t value;
-
-    account_comm_map::iterator ac = account_comms.find(xact->account);
-    if (ac != account_comms.end()) {
-      commodity_t * default_commodity = (*ac).second;
-
-      curr_quant.set_commodity(*default_commodity);
-      value = curr_quant.round(default_commodity->precision);
-
-      if (curr_value.commodity() == *default_commodity)
-	curr_value = value;
-    } else {
-      value = curr_quant;
-    }
-
-    xact->amount = value;
-    if (value != curr_value)
-      xact->cost = new amount_t(curr_value);
     break;
   }
 
@@ -288,18 +337,24 @@ bool gnucash_parser_t::test(std::istream& in) const
 {
   char buf[5];
   in.read(buf, 5);
+  in.clear();
   in.seekg(0, std::ios::beg);
 
   return std::strncmp(buf, "<?xml", 5) == 0;
 }
 
 unsigned int gnucash_parser_t::parse(std::istream&	 in,
+				     config_t&           config,
 				     journal_t *	 journal,
 				     account_t *	 master,
 				     const std::string * original_file)
 {
   char buf[BUFSIZ];
 
+  // This is the date format used by Gnucash, so override whatever the
+  // user specified.
+  date_t::input_format = "%Y-%m-%d %H:%M:%S %z";
+
   count		 = 0;
   action	 = NO_ACTION;
   curr_journal	 = journal;
@@ -308,37 +363,40 @@ unsigned int gnucash_parser_t::parse(std::istream&	 in,
   curr_entry	 = NULL;
   curr_comm	 = NULL;
   entry_comm	 = NULL;
+  curr_state	 = transaction_t::UNCLEARED;
+
+  instreamp = &in;
+  path	    = original_file ? *original_file : "<gnucash>";
+  src_idx   = journal->sources.size() - 1;
 
   // GnuCash uses the USD commodity without defining it, which really
   // means $.
-  commodity_t * usd;
-  usd = new commodity_t("$", 2, COMMODITY_STYLE_THOUSANDS);
-  commodity_t::add_commodity(usd, "$");
-  usd = new commodity_t("$", 2, COMMODITY_STYLE_THOUSANDS);
-  commodity_t::add_commodity(usd, "USD");
+  commodity_t * usd = commodity_t::find_or_create("$");
+  usd->set_precision(2);
+  usd->add_flags(COMMODITY_STYLE_THOUSANDS);
 
-  unsigned int offset = 2;
-  XML_Parser   parser = XML_ParserCreate(NULL);
-  current_parser = parser;
+  offset = 2;
+  parser = current_parser = XML_ParserCreate(NULL);
 
   XML_SetElementHandler(parser, startElement, endElement);
   XML_SetCharacterDataHandler(parser, dataHandler);
 
-  while (! in.eof()) {
+  while (in.good() && ! in.eof()) {
+    beg_pos  = in.tellg();
+    beg_line = (XML_GetCurrentLineNumber(parser) - offset) + 1;
+
     in.getline(buf, BUFSIZ - 1);
     std::strcat(buf, "\n");
     if (! XML_Parse(parser, buf, std::strlen(buf), in.eof())) {
       unsigned long line = XML_GetCurrentLineNumber(parser) - offset++;
       const char *  msg  = XML_ErrorString(XML_GetErrorCode(parser));
       XML_ParserFree(parser);
-      throw parse_error(original_file ? *original_file : "<gnucash>", line,
-			msg);
+      throw new parse_error(msg);
     }
 
     if (! have_error.empty()) {
       unsigned long line = XML_GetCurrentLineNumber(parser) - offset++;
-      parse_error err(original_file ? *original_file : "<gnucash>", line,
-		      have_error);
+      parse_error err(have_error);
       std::cerr << "Error: " << err.what() << std::endl;
       have_error = "";
     }
@@ -353,22 +411,3 @@ unsigned int gnucash_parser_t::parse(std::istream&	 in,
 }
 
 } // namespace ledger
-
-#ifdef USE_BOOST_PYTHON
-
-#include <boost/python.hpp>
-
-using namespace boost::python;
-using namespace ledger;
-
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(gnucash_parse_overloads,
-				       gnucash_parser_t::parse, 2, 4)
-
-void export_gnucash() {
-  class_< gnucash_parser_t, bases<parser_t> > ("GnucashParser")
-    .def("test", &gnucash_parser_t::test)
-    .def("parse", &gnucash_parser_t::parse, gnucash_parse_overloads())
-    ;
-}
-
-#endif // USE_BOOST_PYTHON
