@@ -10,12 +10,6 @@
 
 namespace ledger {
 
-#define MAX_LINE 1024
-
-static char         line[MAX_LINE + 1];
-static std::string  path;
-static unsigned int linenum;
-
 typedef std::map<const std::string, account_t *>  accounts_map;
 typedef std::pair<const std::string, account_t *> accounts_pair;
 
@@ -43,8 +37,8 @@ int ofx_proc_account_cb(struct OfxAccountData data, void * account_data)
   ofx_accounts.insert(accounts_pair(data.account_id, account));
 
   if (data.currency_valid) {
-    commodity_t * commodity = commodity_t::find_commodity(data.currency, true);
-    commodity->flags |= COMMODITY_STYLE_SUFFIXED | COMMODITY_STYLE_SEPARATED;
+    commodity_t * commodity = commodity_t::find_or_create(data.currency);
+    commodity->add_flags(COMMODITY_STYLE_SUFFIXED | COMMODITY_STYLE_SEPARATED);
 
     commodities_map::iterator i = ofx_account_currencies.find(data.account_id);
     if (i == ofx_account_currencies.end())
@@ -75,8 +69,7 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data,
   assert(ac != ofx_account_currencies.end());
   commodity_t * default_commodity = (*ac).second;
 
-  std::string buf;
-  std::ostringstream stream(buf);
+  std::ostringstream stream;
   stream << - data.units;
 
   // jww (2005-02-09): what if the amount contains fees?
@@ -84,24 +77,24 @@ int ofx_proc_transaction_cb(struct OfxTransactionData data,
   if (data.unique_id_valid) {
     commodities_map::iterator s = ofx_securities.find(data.unique_id);
     assert(s != ofx_securities.end());
-    xact->amount = stream.str() + " " + (*s).second->symbol;
+    xact->amount = stream.str() + " " + (*s).second->base_symbol();
   } else {
-    xact->amount = stream.str() + " " + default_commodity->symbol;
+    xact->amount = stream.str() + " " + default_commodity->base_symbol();
   }
 
   if (data.unitprice_valid && data.unitprice != 1.0) {
-    std::ostringstream cstream(buf);
+    std::ostringstream cstream;
     stream << - data.unitprice;
-    xact->cost = new amount_t(stream.str() + " " + default_commodity->symbol);
+    xact->cost = new amount_t(stream.str() + " " + default_commodity->base_symbol());
   }
 
   DEBUG_PRINT("ledger.ofx.parse", "xact " << xact->amount
 	      << " from " << *xact->account);
 
   if (data.date_initiated_valid)
-    entry->date = data.date_initiated;
+    entry->_date = data.date_initiated;
   else if (data.date_posted_valid)
-    entry->date = data.date_posted;
+    entry->_date = data.date_posted;
 
   if (data.check_number_valid)
     entry->code = data.check_number;
@@ -144,14 +137,14 @@ int ofx_proc_security_cb(struct OfxSecurityData data, void * security_data)
   else
     return -1;
 
-  commodity_t * commodity = commodity_t::find_commodity(symbol, true);
-  commodity->flags |= COMMODITY_STYLE_SUFFIXED | COMMODITY_STYLE_SEPARATED;
+  commodity_t * commodity = commodity_t::find_or_create(symbol);
+  commodity->add_flags(COMMODITY_STYLE_SUFFIXED | COMMODITY_STYLE_SEPARATED);
 
   if (data.secname_valid)
-    commodity->name = data.secname;
+    commodity->set_name(data.secname);
 
   if (data.memo_valid)
-    commodity->note = data.memo;
+    commodity->set_note(data.memo);
 
   commodities_map::iterator i = ofx_securities.find(data.unique_id);
   if (i == ofx_securities.end()) {
@@ -178,10 +171,12 @@ bool ofx_parser_t::test(std::istream& in) const
 
   in.getline(buf, 79);
   if (std::strncmp(buf, "OFXHEADER", 9) == 0) {
+    in.clear();
     in.seekg(0, std::ios::beg);
     return true;
   }
   else if (std::strncmp(buf, "<?xml", 5) != 0) {
+    in.clear();
     in.seekg(0, std::ios::beg);
     return false;
   }
@@ -189,15 +184,18 @@ bool ofx_parser_t::test(std::istream& in) const
   in.getline(buf, 79);
   if (std::strncmp(buf, "<?OFX", 5) != 0 &&
       std::strncmp(buf, "<?ofx", 5) != 0) {
+    in.clear();
     in.seekg(0, std::ios::beg);
     return false;
   }
 
+  in.clear();
   in.seekg(0, std::ios::beg);
   return true;
 }
 
 unsigned int ofx_parser_t::parse(std::istream&	     in,
+				 config_t&           config,
 				 journal_t *	     journal,
 				 account_t *	     master,
 				 const std::string * original_file)
@@ -226,22 +224,3 @@ unsigned int ofx_parser_t::parse(std::istream&	     in,
 }
 
 } // namespace ledger
-
-#ifdef USE_BOOST_PYTHON
-
-#include <boost/python.hpp>
-
-using namespace boost::python;
-using namespace ledger;
-
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(ofx_parse_overloads,
-				       ofx_parser_t::parse, 2, 4)
-
-void export_ofx() {
-  class_< ofx_parser_t, bases<parser_t> > ("OfxParser")
-    .def("test", &ofx_parser_t::test)
-    .def("parse", &ofx_parser_t::parse, ofx_parse_overloads())
-    ;
-}
-
-#endif // USE_BOOST_PYTHON
