@@ -114,19 +114,8 @@ unsigned int parse_ledger_data(config_t&   config,
   DEBUG_PRINT("ledger.config.cache",
 	      "3. use_cache = " << config.use_cache);
 
-  if (! config.init_file.empty() &&
-      access(config.init_file.c_str(), R_OK) != -1) {
-    if (parse_journal_file(config.init_file, config, journal) ||
-	journal->auto_entries.size() > 0 ||
-	journal->period_entries.size() > 0)
-      throw new error(std::string("Entries found in initialization file '") +
-		      config.init_file + "'");
-
-    journal->sources.pop_front(); // remove init file
-  }
-
   if (config.use_cache && ! config.cache_file.empty() &&
-      ! config.data_file.empty()) {
+      config.cache_file != "<none>" && ! config.data_file.empty()) {
     DEBUG_PRINT("ledger.config.cache",
 		"using_cache " << config.cache_file);
     config.cache_dirty = true;
@@ -166,16 +155,21 @@ unsigned int parse_ledger_data(config_t&   config,
 		"rejected cache, parsing " << config.data_file);
     if (config.data_file == "-") {
       config.use_cache = false;
-      journal->sources.push_back("<stdin>");
-#if 0
-      // jww (2006-03-23): Why doesn't XML work on stdin?
-      if (xml_parser && std::cin.peek() == '<')
-	entry_count += xml_parser->parse(std::cin, config, journal,
-					 acct);
-      else if (stdin_parser)
-#endif
-	entry_count += stdin_parser->parse(std::cin, config,
-					   journal, acct);
+      journal->sources.push_back("/dev/stdin");
+
+      std::ostringstream buffer;
+
+      while (std::cin.good() && ! std::cin.eof()) {
+	static char line[8192];
+	std::cin.read(line, 8192);
+	std::streamsize count = std::cin.gcount();
+	buffer.write(line, count);
+      }
+      buffer.flush();
+
+      std::istringstream buf_in(buffer.str());
+
+      entry_count += parse_journal(buf_in, config, journal, acct);
     }
     else if (access(config.data_file.c_str(), R_OK) != -1) {
       entry_count += parse_journal_file(config.data_file, config,
