@@ -23,10 +23,10 @@ struct item_handler {
   item_handler(item_handler * _handler) : handler(_handler) {
     DEBUG_PRINT("ledger.memory.ctors", "ctor item_handler<T>");
   }
-
   virtual ~item_handler() {
     DEBUG_PRINT("ledger.memory.dtors", "dtor item_handler<T>");
   }
+
   virtual void flush() {
     if (handler)
       handler->flush();
@@ -56,8 +56,8 @@ bool compare_items<T>::operator()(const T * left, const T * right)
 
   value_t left_result;
   value_t right_result;
-  sort_order->compute(left_result, details_t(*left));
-  sort_order->compute(right_result, details_t(*right));
+  guarded_compute(sort_order, left_result, details_t(*left));
+  guarded_compute(sort_order, right_result, details_t(*right));
 
   return left_result < right_result;
 }
@@ -80,29 +80,63 @@ bool compare_items<account_t>::operator()(const account_t * left,
 #define TRANSACTION_DISPLAYED  0x0008
 #define TRANSACTION_NO_TOTAL   0x0010
 #define TRANSACTION_SORT_CALC  0x0020
-#define TRANSACTION_COMPOSITE  0x0040
+#define TRANSACTION_COMPOUND   0x0040
 #define TRANSACTION_MATCHES    0x0080
 
 struct transaction_xdata_t
 {
   value_t	 total;
   value_t	 sort_value;
-  value_t	 composite_amount;
+  value_t	 value;
   unsigned int   index;
   unsigned short dflags;
-  std::time_t    date;
+  datetime_t     date;
+  account_t *    account;
   void *         ptr;
 
-  transaction_xdata_t() : index(0), dflags(0), date(0), ptr(0) {}
+  transactions_list * component_xacts;
+
+  transaction_xdata_t()
+    : index(0), dflags(0),
+      account(NULL), ptr(NULL), component_xacts(NULL) {
+    DEBUG_PRINT("ledger.memory.ctors", "ctor transaction_xdata_t " << this);
+  }
+
+  ~transaction_xdata_t() {
+    DEBUG_PRINT("ledger.memory.dtors", "dtor transaction_xdata_t " << this);
+    if (component_xacts)
+      delete component_xacts;
+  }
+
+  void remember_xact(transaction_t& xact) {
+    if (! component_xacts)
+      component_xacts = new transactions_list;
+    component_xacts->push_back(&xact);
+  }
+
+  bool have_component_xacts() const {
+    return component_xacts != NULL && ! component_xacts->empty();
+  }
+
+  void copy_component_xacts(transactions_list& xacts) {
+    for (transactions_list::const_iterator i = xacts.begin();
+	 i != xacts.end();
+	 i++)
+      remember_xact(**i);
+  }
+
+  void walk_component_xacts(item_handler<transaction_t>& handler) const {
+    for (transactions_list::const_iterator i = component_xacts->begin();
+	 i != component_xacts->end();
+	 i++)
+      handler(**i);
+  }
 };
 
 inline bool transaction_has_xdata(const transaction_t& xact) {
   return xact.data != NULL;
 }
 
-extern std::list<transaction_xdata_t> transactions_xdata;
-extern std::list<void **>	      transactions_xdata_ptrs;
-
 inline transaction_xdata_t& transaction_xdata_(const transaction_t& xact) {
   return *((transaction_xdata_t *) xact.data);
 }
@@ -110,6 +144,19 @@ inline transaction_xdata_t& transaction_xdata_(const transaction_t& xact) {
 transaction_xdata_t& transaction_xdata(const transaction_t& xact);
 void add_transaction_to(const transaction_t& xact, value_t& value);
 
+inline account_t * xact_account(transaction_t& xact) {
+  if (xact.data) {
+    account_t * account = transaction_xdata(xact).account;
+    if (account)
+      return account;
+  }
+  return xact.account;
+}
+
+inline const account_t * xact_account(const transaction_t& xact) {
+  return xact_account(const_cast<transaction_t&>(xact));
+}
+
 //////////////////////////////////////////////////////////////////////
 
 inline void walk_transactions(transactions_list::iterator begin,
@@ -136,8 +183,6 @@ inline void walk_entries(entries_list& list,
   walk_entries(list.begin(), list.end(), handler);
 }
 
-void clear_transactions_xdata();
-
 //////////////////////////////////////////////////////////////////////
 
 class ignore_transactions : public item_handler<transaction_t>
@@ -146,6 +191,17 @@ class ignore_transactions : public item_handler<transaction_t>
   virtual void operator()(transaction_t& xact) {}
 };
 
+class clear_transaction_xdata : public item_handler<transaction_t>
+{
+ public:
+  virtual void operator()(transaction_t& xact) {
+    if (xact.data) {
+      delete (transaction_xdata_t *) xact.data;
+      xact.data = NULL;
+    }
+  }
+};
+
 class truncate_entries : public item_handler<transaction_t>
 {
   int head_count;
@@ -193,39 +249,66 @@ class sort_transactions : public item_handler<transaction_t>
 
   transactions_deque   transactions;
   const value_expr_t * sort_order;
-  bool                 allocated;
 
  public:
   sort_transactions(item_handler<transaction_t> * handler,
 		    const value_expr_t * _sort_order)
     : item_handler<transaction_t>(handler),
-      sort_order(_sort_order), allocated(false) {}
+      sort_order(_sort_order->acquire()) {}
 
   sort_transactions(item_handler<transaction_t> * handler,
 		    const std::string& _sort_order)
-    : item_handler<transaction_t>(handler), allocated(false) {
-    try {
-      sort_order = parse_value_expr(_sort_order);
-      allocated  = true;
-    }
-    catch (value_expr_error& err) {
-      throw value_expr_error(std::string("In sort string '") + _sort_order +
-			     "': " + err.what());
-    }
+    : item_handler<transaction_t>(handler) {
+    assert(! _sort_order.empty());
+    sort_order = parse_value_expr(_sort_order)->acquire();
   }
 
   virtual ~sort_transactions() {
     assert(sort_order);
-    if (allocated)
-      delete sort_order;
+    sort_order->release();
+  }
+
+  virtual void post_accumulated_xacts();
+
+  virtual void flush() {
+    post_accumulated_xacts();
+    item_handler<transaction_t>::flush();
   }
 
-  virtual void flush();
   virtual void operator()(transaction_t& xact) {
     transactions.push_back(&xact);
   }
 };
 
+class sort_entries : public item_handler<transaction_t>
+{
+  sort_transactions sorter;
+  entry_t * last_entry;
+
+ public:
+  sort_entries(item_handler<transaction_t> * handler,
+	       const value_expr_t * _sort_order)
+    : sorter(handler, _sort_order) {}
+
+  sort_entries(item_handler<transaction_t> * handler,
+	       const std::string& _sort_order)
+    : sorter(handler, _sort_order) {}
+
+  virtual void flush() {
+    sorter.flush();
+    item_handler<transaction_t>::flush();
+  }
+
+  virtual void operator()(transaction_t& xact) {
+    if (last_entry && xact.entry != last_entry)
+      sorter.post_accumulated_xacts();
+
+    sorter(xact);
+
+    last_entry = xact.entry;
+  }
+};
+
 class filter_transactions : public item_handler<transaction_t>
 {
   item_predicate<transaction_t> pred;
@@ -306,6 +389,22 @@ class collapse_transactions : public item_handler<transaction_t>
   virtual void operator()(transaction_t& xact);
 };
 
+class component_transactions : public item_handler<transaction_t>
+{
+  item_predicate<transaction_t> pred;
+
+ public:
+  component_transactions(item_handler<transaction_t> * handler,
+			 const value_expr_t * predicate)
+    : item_handler<transaction_t>(handler), pred(predicate) {}
+
+  component_transactions(item_handler<transaction_t> * handler,
+			 const std::string& predicate)
+    : item_handler<transaction_t>(handler), pred(predicate) {}
+
+  virtual void operator()(transaction_t& xact);
+};
+
 class related_transactions : public item_handler<transaction_t>
 {
   transactions_list transactions;
@@ -348,41 +447,55 @@ class changed_value_transactions : public item_handler<transaction_t>
 
   virtual void flush() {
     if (last_xact) {
-      output_diff(now);
+      output_diff(datetime_t::now);
       last_xact = NULL;
     }
     item_handler<transaction_t>::flush();
   }
 
-  void output_diff(const std::time_t current);
+  void output_diff(const datetime_t& current);
 
   virtual void operator()(transaction_t& xact);
 };
 
 class subtotal_transactions : public item_handler<transaction_t>
 {
-  typedef std::map<account_t *, value_t>  values_map;
-  typedef std::pair<account_t *, value_t> values_pair;
+  struct acct_value_t {
+    account_t *	account;
+    value_t	value;
+
+    transactions_list components;
+
+    acct_value_t(account_t * a) : account(a) {}
+    acct_value_t(account_t * a, value_t& v) : account(a), value(v) {}
+    acct_value_t(const acct_value_t& av)
+      : account(av.account), value(av.value) {}
+  };
+
+  typedef std::map<std::string, acct_value_t>  values_map;
+  typedef std::pair<std::string, acct_value_t> values_pair;
 
  protected:
   values_map values;
+  bool       remember_components;
 
   std::list<entry_t>       entry_temps;
   std::list<transaction_t> xact_temps;
 
  public:
-  std::time_t start;
-  std::time_t finish;
+  datetime_t start;
+  datetime_t finish;
 
-  subtotal_transactions(item_handler<transaction_t> * handler)
-    : item_handler<transaction_t>(handler), start(0), finish(0) {}
+  subtotal_transactions(item_handler<transaction_t> * handler,
+			bool _remember_components = false)
+    : item_handler<transaction_t>(handler),
+      remember_components(_remember_components) {}
 #ifdef DEBUG_ENABLED
   subtotal_transactions(const subtotal_transactions&) {
     assert(0);
   }
 #endif
-
-  ~subtotal_transactions() {
+  virtual ~subtotal_transactions() {
     clear_entries_transactions(entry_temps);
   }
 
@@ -396,42 +509,34 @@ class subtotal_transactions : public item_handler<transaction_t>
   virtual void operator()(transaction_t& xact);
 };
 
+class interval_expr_error : public error {
+ public:
+  interval_expr_error(const std::string& reason,
+		      error_context * ctxt = NULL) throw()
+    : error(reason, ctxt) {}
+  virtual ~interval_expr_error() throw() {}
+};
+
 class interval_transactions : public subtotal_transactions
 {
   interval_t      interval;
   transaction_t * last_xact;
   bool            started;
 
-  item_handler<transaction_t> * sorter;
-
  public:
   interval_transactions(item_handler<transaction_t> * _handler,
-			const interval_t&    _interval,
-			const value_expr_t * sort_order = NULL)
-    : subtotal_transactions(_handler), interval(_interval),
-      last_xact(NULL), started(false), sorter(NULL) {
-    if (sort_order) {
-      sorter = new sort_transactions(handler, sort_order);
-      handler = sorter;
-    }
-  }
+			const interval_t& _interval,
+			bool remember_components = false)
+    : subtotal_transactions(_handler, remember_components),
+      interval(_interval), last_xact(NULL), started(false) {}
+
   interval_transactions(item_handler<transaction_t> * _handler,
 			const std::string& _interval,
-			const std::string& sort_order = "")
-    : subtotal_transactions(_handler), interval(_interval),
-      last_xact(NULL), started(false), sorter(NULL) {
-    if (! sort_order.empty()) {
-      sorter = new sort_transactions(handler, sort_order);
-      handler = sorter;
-    }
-  }
+			bool remember_components = false)
+    : subtotal_transactions(_handler, remember_components),
+      interval(_interval), last_xact(NULL), started(false) {}
 
-  virtual ~interval_transactions() {
-    if (sorter)
-      delete sorter;
-  }
-
-  void report_subtotal(const std::time_t moment = 0);
+  void report_subtotal(const datetime_t& moment = datetime_t());
 
   virtual void flush() {
     if (last_xact)
@@ -447,11 +552,13 @@ class by_payee_transactions : public item_handler<transaction_t>
   typedef std::pair<std::string, subtotal_transactions *> payee_subtotals_pair;
 
   payee_subtotals_map payee_subtotals;
+  bool remember_components;
 
  public:
-  by_payee_transactions(item_handler<transaction_t> * handler)
-    : item_handler<transaction_t>(handler) {}
-
+  by_payee_transactions(item_handler<transaction_t> * handler,
+			bool _remember_components = false)
+    : item_handler<transaction_t>(handler),
+      remember_components(_remember_components) {}
   virtual ~by_payee_transactions();
 
   virtual void flush();
@@ -474,18 +581,34 @@ class set_comm_as_payee : public item_handler<transaction_t>
   virtual void operator()(transaction_t& xact);
 };
 
+class set_code_as_payee : public item_handler<transaction_t>
+{
+  std::list<entry_t>       entry_temps;
+  std::list<transaction_t> xact_temps;
+
+ public:
+  set_code_as_payee(item_handler<transaction_t> * handler)
+    : item_handler<transaction_t>(handler) {}
+
+  ~set_code_as_payee() {
+    clear_entries_transactions(entry_temps);
+  }
+
+  virtual void operator()(transaction_t& xact);
+};
+
 class dow_transactions : public subtotal_transactions
 {
   transactions_list days_of_the_week[7];
 
  public:
-  dow_transactions(item_handler<transaction_t> * handler)
-    : subtotal_transactions(handler) {}
+  dow_transactions(item_handler<transaction_t> * handler,
+		   bool remember_components = false)
+    : subtotal_transactions(handler, remember_components) {}
 
   virtual void flush();
   virtual void operator()(transaction_t& xact) {
-    struct std::tm * desc = std::localtime(&xact.entry->date);
-    days_of_the_week[desc->tm_wday].push_back(&xact);
+    days_of_the_week[xact.date().wday()].push_back(&xact);
   }
 };
 
@@ -525,7 +648,7 @@ class budget_transactions : public generate_transactions
 		      unsigned long _flags = BUDGET_BUDGETED)
     : generate_transactions(handler), flags(_flags) {}
 
-  void report_budget_items(const std::time_t moment);
+  void report_budget_items(const datetime_t& moment);
 
   virtual void operator()(transaction_t& xact);
 };
@@ -577,9 +700,6 @@ inline bool account_has_xdata(const account_t& account) {
   return account.data != NULL;
 }
 
-extern std::list<account_xdata_t> accounts_xdata;
-extern std::list<void **>	  accounts_xdata_ptrs;
-
 inline account_xdata_t& account_xdata_(const account_t& account) {
   return *((account_xdata_t *) account.data);
 }
@@ -588,6 +708,17 @@ account_xdata_t& account_xdata(const account_t& account);
 
 //////////////////////////////////////////////////////////////////////
 
+class clear_account_xdata : public item_handler<account_t>
+{
+ public:
+  virtual void operator()(account_t& acct) {
+    if (acct.data) {
+      delete (account_xdata_t *) acct.data;
+      acct.data = NULL;
+    }
+  }
+};
+
 void sum_accounts(account_t& account);
 
 typedef std::deque<account_t *> accounts_deque;
@@ -602,18 +733,19 @@ void walk_accounts(account_t&		    account,
 		   item_handler<account_t>& handler,
 		   const std::string&       sort_string);
 
-void clear_accounts_xdata();
-
-inline void clear_all_xdata() {
-  clear_transactions_xdata();
-  clear_accounts_xdata();
-}
-
 //////////////////////////////////////////////////////////////////////
 
 void walk_commodities(commodities_map& commodities,
 		      item_handler<transaction_t>& handler);
 
+inline void clear_journal_xdata(journal_t * journal) {
+  clear_transaction_xdata xact_cleaner;
+  walk_entries(journal->entries, xact_cleaner);
+
+  clear_account_xdata acct_cleaner;
+  walk_accounts(*journal->master, acct_cleaner);
+}
+
 } // namespace ledger
 
 #endif // _WALK_H
