@@ -8,11 +8,17 @@
 #include <cstring>
 
 extern "C" {
-#include <xmlparse.h>           // expat XML parser
+#if defined(HAVE_EXPAT)
+#include <expat.h>           // expat XML parser
+#elif defined(HAVE_XMLPARSE)
+#include <xmlparse.h>        // expat XML parser
+#endif
 }
 
 namespace ledger {
 
+#if defined(HAVE_EXPAT) || defined(HAVE_XMLPARSE)
+
 static XML_Parser    current_parser;
 static unsigned int  count;
 
@@ -21,6 +27,8 @@ static entry_t *     curr_entry;
 static commodity_t * curr_comm;
 static std::string   comm_flags;
 
+static transaction_t::state_t curr_state;
+
 static std::string   data;
 static bool          ignore;
 static std::string   have_error;
@@ -33,10 +41,13 @@ static void startElement(void *userData, const char *name, const char **attrs)
   if (std::strcmp(name, "entry") == 0) {
     assert(! curr_entry);
     curr_entry = new entry_t;
+    curr_state = transaction_t::UNCLEARED;
   }
   else if (std::strcmp(name, "transaction") == 0) {
     assert(curr_entry);
     curr_entry->add_transaction(new transaction_t);
+    if (curr_state != transaction_t::UNCLEARED)
+      curr_entry->transactions.back()->state = curr_state;
   }
   else if (std::strcmp(name, "commodity") == 0) {
     if (std::string(attrs[0]) == "flags")
@@ -72,50 +83,76 @@ static void endElement(void *userData, const char *name)
     curr_entry = NULL;
   }
   else if (std::strcmp(name, "en:date") == 0) {
-    quick_parse_date(data.c_str(), &curr_entry->date);
+    curr_entry->_date = data;
   }
-  else if (std::strcmp(name, "en:cleared") == 0) {
-    curr_entry->state = entry_t::CLEARED;
-  }
-  else if (std::strcmp(name, "en:pending") == 0) {
-    curr_entry->state = entry_t::PENDING;
+  else if (std::strcmp(name, "en:date_eff") == 0) {
+    curr_entry->_date_eff = data;
   }
   else if (std::strcmp(name, "en:code") == 0) {
     curr_entry->code = data;
   }
+  else if (std::strcmp(name, "en:cleared") == 0) {
+    curr_state = transaction_t::CLEARED;
+  }
+  else if (std::strcmp(name, "en:pending") == 0) {
+    curr_state = transaction_t::PENDING;
+  }
   else if (std::strcmp(name, "en:payee") == 0) {
     curr_entry->payee = data;
   }
   else if (std::strcmp(name, "tr:account") == 0) {
     curr_entry->transactions.back()->account = curr_journal->find_account(data);
   }
+  else if (std::strcmp(name, "tr:cleared") == 0) {
+    curr_entry->transactions.back()->state = transaction_t::CLEARED;
+  }
+  else if (std::strcmp(name, "tr:pending") == 0) {
+    curr_entry->transactions.back()->state = transaction_t::PENDING;
+  }
   else if (std::strcmp(name, "tr:virtual") == 0) {
     curr_entry->transactions.back()->flags |= TRANSACTION_VIRTUAL;
   }
   else if (std::strcmp(name, "tr:generated") == 0) {
     curr_entry->transactions.back()->flags |= TRANSACTION_AUTO;
   }
-  else if (std::strcmp(name, "commodity") == 0) {
+  else if (std::strcmp(name, "symbol") == 0) {
     assert(! curr_comm);
-    curr_comm = commodity_t::find_commodity(data, true);
-    curr_comm->flags |= COMMODITY_STYLE_SUFFIXED;
-    if (! comm_flags.empty())
-      for (std::string::size_type i = 0, l = comm_flags.length(); i < l; i++)
+    curr_comm = commodity_t::find_or_create(data);
+    assert(curr_comm);
+    curr_comm->add_flags(COMMODITY_STYLE_SUFFIXED);
+    if (! comm_flags.empty()) {
+      for (std::string::size_type i = 0, l = comm_flags.length(); i < l; i++) {
 	switch (comm_flags[i]) {
-	case 'P': curr_comm->flags &= ~COMMODITY_STYLE_SUFFIXED; break;
-	case 'S': curr_comm->flags |= COMMODITY_STYLE_SEPARATED; break;
-	case 'T': curr_comm->flags |= COMMODITY_STYLE_THOUSANDS; break;
-	case 'E': curr_comm->flags |= COMMODITY_STYLE_EUROPEAN; break;
+	case 'P': curr_comm->drop_flags(COMMODITY_STYLE_SUFFIXED); break;
+	case 'S': curr_comm->add_flags(COMMODITY_STYLE_SEPARATED); break;
+	case 'T': curr_comm->add_flags(COMMODITY_STYLE_THOUSANDS); break;
+	case 'E': curr_comm->add_flags(COMMODITY_STYLE_EUROPEAN); break;
 	}
+      }
+    }
   }
+#if 0
+  // jww (2006-03-02): !!!
+  else if (std::strcmp(name, "price") == 0) {
+    assert(curr_comm);
+    amount_t * price = new amount_t(data);
+    std::ostringstream symstr;
+    symstr << curr_comm->symbol << " {" << *price << "}";
+    commodity_t * priced_comm =
+      commodity_t::find_commodity(symstr.str(), true);
+    priced_comm->price = price;
+    priced_comm->base = curr_comm;
+    curr_comm = priced_comm;
+  }
+#endif
   else if (std::strcmp(name, "quantity") == 0) {
     curr_entry->transactions.back()->amount.parse(data);
     if (curr_comm) {
       std::string::size_type i = data.find('.');
       if (i != std::string::npos) {
 	int precision = data.length() - i - 1;
-	if (precision > curr_comm->precision)
-	  curr_comm->precision = precision;
+	if (precision > curr_comm->precision())
+	  curr_comm->set_precision(precision);
       }
       curr_entry->transactions.back()->amount.set_commodity(*curr_comm);
       curr_comm = NULL;
@@ -138,24 +175,28 @@ bool xml_parser_t::test(std::istream& in) const
 
   in.getline(buf, 79);
   if (std::strncmp(buf, "<?xml", 5) != 0) {
+    in.clear();
     in.seekg(0, std::ios::beg);
     return false;
   }
 
   in.getline(buf, 79);
   if (! std::strstr(buf, "<ledger")) {
+    in.clear();
     in.seekg(0, std::ios::beg);
     return false;
   }
 
+  in.clear();
   in.seekg(0, std::ios::beg);
   return true;
 }
 
-unsigned int xml_parser_t::parse(std::istream&	 in,
-				     journal_t *	 journal,
-				     account_t *	 master,
-				     const std::string * original_file)
+unsigned int xml_parser_t::parse(std::istream&	     in,
+				 config_t&           config,
+				 journal_t *	     journal,
+				 account_t *	     master,
+				 const std::string * original_file)
 {
   char buf[BUFSIZ];
 
@@ -182,14 +223,12 @@ unsigned int xml_parser_t::parse(std::istream&	 in,
     catch (const std::exception& err) {
       unsigned long line = XML_GetCurrentLineNumber(parser) - offset++;
       XML_ParserFree(parser);
-      throw parse_error(original_file ? *original_file : "<xml>", line,
-			err.what());
+      throw new parse_error(err.what());
     }
 
     if (! have_error.empty()) {
       unsigned long line = XML_GetCurrentLineNumber(parser) - offset++;
-      parse_error err(original_file ? *original_file : "<xml>", line,
-		      have_error);
+      parse_error err(have_error);
       std::cerr << "Error: " << err.what() << std::endl;
       have_error = "";
     }
@@ -198,7 +237,7 @@ unsigned int xml_parser_t::parse(std::istream&	 in,
       unsigned long line = XML_GetCurrentLineNumber(parser) - offset++;
       const char *  err  = XML_ErrorString(XML_GetErrorCode(parser));
       XML_ParserFree(parser);
-      throw parse_error(original_file ? *original_file : "<xml>", line, err);
+      throw new parse_error(err);
     }
   }
 
@@ -207,6 +246,8 @@ unsigned int xml_parser_t::parse(std::istream&	 in,
   return count;
 }
 
+#endif // defined(HAVE_EXPAT) || defined(HAVE_XMLPARSE)
+
 void xml_write_amount(std::ostream& out, const amount_t& amount,
 		      const int depth = 0)
 {
@@ -216,11 +257,27 @@ void xml_write_amount(std::ostream& out, const amount_t& amount,
   commodity_t& c = amount.commodity();
   for (int i = 0; i < depth + 2; i++) out << ' ';
   out << "<commodity flags=\"";
-  if (! (c.flags & COMMODITY_STYLE_SUFFIXED)) out << 'P';
-  if (c.flags & COMMODITY_STYLE_SEPARATED)    out << 'S';
-  if (c.flags & COMMODITY_STYLE_THOUSANDS)    out << 'T';
-  if (c.flags & COMMODITY_STYLE_EUROPEAN)     out << 'E';
-  out << "\">" << c.symbol << "</commodity>\n";
+  if (! (c.flags() & COMMODITY_STYLE_SUFFIXED)) out << 'P';
+  if (c.flags() & COMMODITY_STYLE_SEPARATED)    out << 'S';
+  if (c.flags() & COMMODITY_STYLE_THOUSANDS)    out << 'T';
+  if (c.flags() & COMMODITY_STYLE_EUROPEAN)     out << 'E';
+  out << "\">\n";
+  for (int i = 0; i < depth + 4; i++) out << ' ';
+#if 0
+  // jww (2006-03-02): !!!
+  if (c.price) {
+    out << "<symbol>" << c.base->symbol << "</symbol>\n";
+    for (int i = 0; i < depth + 4; i++) out << ' ';
+    out << "<price>\n";
+    xml_write_amount(out, *c.price, depth + 6);
+    for (int i = 0; i < depth + 4; i++) out << ' ';
+    out << "</price>\n";
+  } else {
+    out << "<symbol>" << c.symbol << "</symbol>\n";
+  }
+#endif
+  for (int i = 0; i < depth + 2; i++) out << ' ';
+  out << "</commodity>\n";
 
   for (int i = 0; i < depth + 2; i++) out << ' ';
   out << "<quantity>";
@@ -312,17 +369,14 @@ void output_xml_string(std::ostream& out, const std::string& str)
 
 void format_xml_entries::format_last_entry()
 {
-  char buf[256];
-  std::strftime(buf, 255, format_t::date_format.c_str(),
-		std::localtime(&last_entry->date));
-
   output_stream << "  <entry>\n"
-		<< "    <en:date>" << buf << "</en:date>\n";
+		<< "    <en:date>" << last_entry->_date.to_string("%Y/%m/%d")
+		<< "</en:date>\n";
 
-  if (last_entry->state == entry_t::CLEARED)
-    output_stream << "    <en:cleared/>\n";
-  else if (last_entry->state == entry_t::PENDING)
-    output_stream << "    <en:pending/>\n";
+  if (last_entry->_date_eff)
+    output_stream << "    <en:date_eff>"
+		  << last_entry->_date_eff.to_string("%Y/%m/%d")
+		  << "</en:date_eff>\n";
 
   if (! last_entry->code.empty()) {
     output_stream << "    <en:code>";
@@ -349,6 +403,21 @@ void format_xml_entries::format_last_entry()
 
       output_stream << "      <transaction>\n";
 
+      if ((*i)->_date)
+	output_stream << "        <tr:date>"
+		      << (*i)->_date.to_string("%Y/%m/%d")
+		      << "</tr:date>\n";
+
+      if ((*i)->_date_eff)
+	output_stream << "        <tr:date_eff>"
+		      << (*i)->_date_eff.to_string("%Y/%m/%d")
+		      << "</tr:date_eff>\n";
+
+      if ((*i)->state == transaction_t::CLEARED)
+	output_stream << "        <tr:cleared/>\n";
+      else if ((*i)->state == transaction_t::PENDING)
+	output_stream << "        <tr:pending/>\n";
+
       if ((*i)->flags & TRANSACTION_VIRTUAL)
 	output_stream << "        <tr:virtual/>\n";
       if ((*i)->flags & TRANSACTION_AUTO)
@@ -367,9 +436,9 @@ void format_xml_entries::format_last_entry()
       }
 
       output_stream << "        <tr:amount>\n";
-      if (transaction_xdata_(**i).dflags & TRANSACTION_COMPOSITE)
+      if (transaction_xdata_(**i).dflags & TRANSACTION_COMPOUND)
 	xml_write_value(output_stream,
-			transaction_xdata_(**i).composite_amount, 10);
+			transaction_xdata_(**i).value, 10);
       else
 	xml_write_value(output_stream, value_t((*i)->amount), 10);
       output_stream << "        </tr:amount>\n";
@@ -405,33 +474,3 @@ void format_xml_entries::format_last_entry()
 }
 
 } // namespace ledger
-
-#ifdef USE_BOOST_PYTHON
-
-#include <boost/python.hpp>
-
-using namespace boost::python;
-using namespace ledger;
-
-BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(xml_parse_overloads,
-				       xml_parser_t::parse, 2, 4)
-
-void export_xml() {
-  class_< xml_parser_t, bases<parser_t> > ("XmlParser")
-    .def("test", &xml_parser_t::test)
-    .def("parse", &xml_parser_t::parse, xml_parse_overloads())
-    ;
-
-  typedef
-    pystream_handler_wrap<format_xml_entries, transaction_t, bool>
-    format_xml_entries_wrap;
-
-  class_< format_xml_entries_wrap, bases<item_handler<transaction_t> > >
-    ("FormatXmlEntries",
-     init<PyObject *, bool>()[with_custodian_and_ward<1, 2>()])
-    .def("flush", &format_xml_entries_wrap::flush)
-    .def("__call__", &format_xml_entries_wrap::operator())
-    ;
-}
-
-#endif // USE_BOOST_PYTHON
